@@ -1,13 +1,24 @@
 use std::collections::HashSet;
 
 use crate::utils;
-use proc_macro2::{Span, TokenStream};
+use proc_macro2::{
+  Span,
+  TokenStream,
+};
 use quote::quote;
 #[cfg(not(test))]
 use quote::ToTokens;
-use syn::parse::{Parse, ParseStream};
-use syn::spanned::Spanned;
-use synstructure::{Structure, VariantInfo};
+use syn::{
+  parse::{
+    Parse,
+    ParseStream,
+  },
+  spanned::Spanned,
+};
+use synstructure::{
+  Structure,
+  VariantInfo,
+};
 
 mod kw {
   use syn::custom_keyword;
@@ -32,9 +43,11 @@ impl Parse for MhAttr {
   fn parse(input: ParseStream) -> syn::Result<Self> {
     if input.peek(kw::code) {
       Ok(MhAttr::Code(input.parse()?))
-    } else if input.peek(kw::hasher) {
+    }
+    else if input.peek(kw::hasher) {
       Ok(MhAttr::Hasher(input.parse()?))
-    } else {
+    }
+    else {
       Ok(MhAttr::Digest(input.parse()?))
     }
   }
@@ -43,7 +56,7 @@ impl Parse for MhAttr {
 /// Attributes of the top-level derive.
 #[derive(Debug)]
 enum DeriveAttr {
-  AllocSize(utils::Attr<kw::alloc_size, syn::Type>),
+  AllocSize(utils::Attr<kw::alloc_size, syn::LitInt>),
   NoAllocSizeErrors(kw::no_alloc_size_errors),
 }
 
@@ -51,9 +64,11 @@ impl Parse for DeriveAttr {
   fn parse(input: ParseStream) -> syn::Result<Self> {
     if input.peek(kw::alloc_size) {
       Ok(Self::AllocSize(input.parse()?))
-    } else if input.peek(kw::no_alloc_size_errors) {
+    }
+    else if input.peek(kw::no_alloc_size_errors) {
       Ok(Self::NoAllocSizeErrors(input.parse()?))
-    } else {
+    }
+    else {
       Err(syn::Error::new(input.span(), "unknown attribute"))
     }
   }
@@ -115,7 +130,8 @@ impl<'a> From<&'a VariantInfo<'a>> for Hash {
     let mut digest = None;
     let mut hasher = None;
     for attr in bi.ast().attrs {
-      let attr: Result<utils::Attrs<MhAttr>, _> = syn::parse2(attr.tokens.clone());
+      let attr: Result<utils::Attrs<MhAttr>, _> =
+        syn::parse2(attr.tokens.clone());
       if let Ok(attr) = attr {
         for attr in attr.attrs {
           match attr {
@@ -129,48 +145,50 @@ impl<'a> From<&'a VariantInfo<'a>> for Hash {
 
     let ident = bi.ast().ident.clone();
     let code = code.unwrap_or_else(|| {
-      let msg = "Missing code attribute: e.g. #[mh(code = multihash::SHA3_256)]";
+      let msg =
+        "Missing code attribute: e.g. #[mh(code = multihash::SHA3_256)]";
       #[cfg(test)]
       panic!("{}", msg);
       #[cfg(not(test))]
       proc_macro_error::abort!(ident, msg);
     });
     let hasher = hasher.unwrap_or_else(|| {
-      let msg = "Missing hasher attribute: e.g. #[mh(hasher = multihash::Sha2_256)]";
+      let msg =
+        "Missing hasher attribute: e.g. #[mh(hasher = multihash::Sha2_256)]";
       #[cfg(test)]
       panic!("{}", msg);
       #[cfg(not(test))]
       proc_macro_error::abort!(ident, msg);
     });
     let digest = digest.unwrap_or_else(|| {
-      let msg = "Missing digest atttibute: e.g. #[mh(digest = multihash::Sha2Digest<U32>)]";
+      let msg = "Missing digest atttibute: e.g. #[mh(digest = \
+                 multihash::Sha2Digest<32>)]";
       #[cfg(test)]
       panic!("{}", msg);
       #[cfg(not(test))]
       proc_macro_error::abort!(ident, msg);
     });
-    Self {
-      ident,
-      code,
-      hasher,
-      digest,
-    }
+    Self { ident, code, hasher, digest }
   }
 }
 
 /// Parse top-level enum [#mh()] attributes.
 ///
-/// Returns the `alloc_size` and whether errors regarding to `alloc_size` should be reported or not.
-fn parse_code_enum_attrs(ast: &syn::DeriveInput) -> (syn::Type, bool) {
+/// Returns the `alloc_size` and whether errors regarding to `alloc_size` should
+/// be reported or not.
+fn parse_code_enum_attrs(ast: &syn::DeriveInput) -> (syn::LitInt, bool) {
   let mut alloc_size = None;
   let mut no_alloc_size_errors = false;
 
   for attr in &ast.attrs {
-    let derive_attrs: Result<utils::Attrs<DeriveAttr>, _> = syn::parse2(attr.tokens.clone());
+    let derive_attrs: Result<utils::Attrs<DeriveAttr>, _> =
+      syn::parse2(attr.tokens.clone());
     if let Ok(derive_attrs) = derive_attrs {
       for derive_attr in derive_attrs.attrs {
         match derive_attr {
-          DeriveAttr::AllocSize(alloc_size_attr) => alloc_size = Some(alloc_size_attr.value),
+          DeriveAttr::AllocSize(alloc_size_attr) => {
+            alloc_size = Some(alloc_size_attr.value)
+          }
           DeriveAttr::NoAllocSizeErrors(_) => no_alloc_size_errors = true,
         }
       }
@@ -179,7 +197,8 @@ fn parse_code_enum_attrs(ast: &syn::DeriveInput) -> (syn::Type, bool) {
   match alloc_size {
     Some(alloc_size) => (alloc_size, no_alloc_size_errors),
     None => {
-      let msg = "enum is missing `alloc_size` attribute: e.g. #[mh(alloc_size = U64)]";
+      let msg =
+        "enum is missing `alloc_size` attribute: e.g. #[mh(alloc_size = 64)]";
       #[cfg(test)]
       panic!("{}", msg);
       #[cfg(not(test))]
@@ -190,8 +209,8 @@ fn parse_code_enum_attrs(ast: &syn::DeriveInput) -> (syn::Type, bool) {
 
 /// Return an error if the same code is used several times.
 ///
-/// This only checks for string equality, though this should still catch most errors caused by
-/// copy and pasting.
+/// This only checks for string equality, though this should still catch most
+/// errors caused by copy and pasting.
 fn error_code_duplicates(hashes: &[Hash]) {
   // Use a temporary store to determine whether a certain value is unique or not
   let mut uniq = HashSet::new();
@@ -224,33 +243,14 @@ fn error_code_duplicates(hashes: &[Hash]) {
 #[derive(Debug)]
 struct ParseError(Span);
 
-/// Parse a path containing a `typenum` unsigned integer (e.g. `U64`) into a u64
-fn parse_unsigned_typenum(typenum_path: &syn::Type) -> Result<u64, ParseError> {
-  match typenum_path {
-    syn::Type::Path(type_path) => match type_path.path.segments.last() {
-      Some(path_segment) => {
-        let typenum_ident = &path_segment.ident;
-        let typenum = typenum_ident.to_string();
-        match typenum.as_str().split_at(1) {
-          ("U", byte_size) => byte_size
-            .parse::<u64>()
-            .map_err(|_| ParseError(typenum_ident.span())),
-          _ => Err(ParseError(typenum_ident.span())),
-        }
-      }
-      None => Err(ParseError(type_path.path.span())),
-    },
-    _ => Err(ParseError(typenum_path.span())),
-  }
-}
-
 /// Returns the max size as u64.
 ///
-/// Emits an error if the `#mh(alloc_size)` attribute doesn't contain a valid unsigned integer
-/// `typenum`.
-fn parse_alloc_size_attribute(alloc_size: &syn::Type) -> u64 {
-  parse_unsigned_typenum(alloc_size).unwrap_or_else(|_| {
-    let msg = "`alloc_size` attribute must be a `typenum`, e.g. #[mh(alloc_size = U64)]";
+/// Emits an error if the `#mh(alloc_size)` attribute doesn't contain a valid
+/// unsigned integer.
+fn parse_alloc_size_attribute(alloc_size: &syn::LitInt) -> u64 {
+  alloc_size.base10_parse().unwrap_or_else(|_| {
+    let msg =
+      "`alloc_size` attribute must be an integer, e.g. #[mh(alloc_size = 64)]";
     #[cfg(test)]
     panic!("{}", msg);
     #[cfg(not(test))]
@@ -258,39 +258,42 @@ fn parse_alloc_size_attribute(alloc_size: &syn::Type) -> u64 {
   })
 }
 
-/// Return a warning/error if the specified alloc_size is smaller than the biggest digest
-fn error_alloc_size(hashes: &[Hash], expected_alloc_size_type: &syn::Type) {
-  let expected_alloc_size = parse_alloc_size_attribute(expected_alloc_size_type);
+/// Return a warning/error if the specified alloc_size is smaller than the
+/// biggest digest
+fn error_alloc_size(hashes: &[Hash], expected_alloc_size_type: &syn::LitInt) {
+  let expected_alloc_size =
+    parse_alloc_size_attribute(expected_alloc_size_type);
 
   let maybe_error: Result<(), ParseError> = hashes
         .iter()
         .try_for_each(|hash| {
-            // The digest type must have a size parameter of the shape `U<number>`, else we error.
+            // The digest type must have an integer as size parameter, else we error.
             match hash.digest.segments.last() {
                 Some(path_segment) => match &path_segment.arguments {
                     syn::PathArguments::AngleBracketed(arguments) => match arguments.args.last() {
-                        Some(syn::GenericArgument::Type(path)) => {
-                            match parse_unsigned_typenum(path) {
-                                Ok(max_digest_size) => {
-                                    if max_digest_size > expected_alloc_size {
-                                        let msg = format!("The `#mh(alloc_size) attribute must be bigger than the maximum defined digest size (U{})",
-                                        max_digest_size);
-                                        #[cfg(test)]
-                                        panic!("{}", msg);
-                                        #[cfg(not(test))]
-                                        {
-                                            let digest = &hash.digest.to_token_stream().to_string().replace(" ", "");
-                                            let line = &hash.digest.span().start().line;
-                                            proc_macro_error::emit_error!(
-                                                &expected_alloc_size_type, msg;
-                                                note = "the bigger digest is `{}` at line {}", digest, line;
-                                            );
-                                        }
-                                    }
-                                    Ok(())
-                                },
-                                Err(err) => Err(err),
-                            }
+                      Some(syn::GenericArgument::Const(syn::Expr::Lit(expr_lit))) => match &expr_lit.lit {
+                           syn::Lit::Int(lit_int) => match lit_int.base10_parse::<u64>() {
+                              Ok(max_digest_size) => {
+                                  if max_digest_size > expected_alloc_size {
+                                      let msg = format!("The `#mh(alloc_size) attribute must be bigger than the maximum defined digest size ({})",
+                                      max_digest_size);
+                                      #[cfg(test)]
+                                      panic!("{}", msg);
+                                      #[cfg(not(test))]
+                                      {
+                                          let digest = &hash.digest.to_token_stream().to_string().replace(" ", "");
+                                          let line = &hash.digest.span().start().line;
+                                          proc_macro_error::emit_error!(
+                                              &expected_alloc_size_type, msg;
+                                              note = "the bigger digest is `{}` at line {}", digest, line;
+                                          );
+                                      }
+                                  }
+                                  Ok(())
+                              },
+                              _ => Err(ParseError(lit_int.span())),
+                           },
+                           _ => Err(ParseError(expr_lit.span())),
                         },
                         _ => Err(ParseError(arguments.args.span())),
                     },
@@ -301,7 +304,7 @@ fn error_alloc_size(hashes: &[Hash], expected_alloc_size_type: &syn::Type) {
         });
 
   if let Err(_error) = maybe_error {
-    let msg = "Invalid byte size. It must be a unsigned integer typenum, e.g. `U32`";
+    let msg = "Invalid byte size. It must be a unsigned integer, e.g. `32`";
     #[cfg(test)]
     panic!("{}", msg);
     #[cfg(not(test))]
@@ -320,7 +323,7 @@ pub fn multihash(s: Structure) -> TokenStream {
     }
   };
   let code_enum = &s.ast().ident;
-  let (alloc_size, no_alloc_size_errors) = parse_code_enum_attrs(s.ast());
+  let (alloc_size, no_alloc_size_errors) = parse_code_enum_attrs(&s.ast());
   let hashes: Vec<_> = s.variants().iter().map(Hash::from).collect();
 
   error_code_duplicates(&hashes);
@@ -329,9 +332,7 @@ pub fn multihash(s: Structure) -> TokenStream {
     error_alloc_size(&hashes, &alloc_size);
   }
 
-  let params = Params {
-    code_enum: code_enum.clone(),
-  };
+  let params = Params { code_enum: code_enum.clone() };
 
   let code_into_u64 = hashes.iter().map(|h| h.code_into_u64(&params));
   let code_from_u64 = hashes.iter().map(|h| h.code_from_u64());
@@ -342,9 +343,7 @@ pub fn multihash(s: Structure) -> TokenStream {
       /// A Multihash with the same allocated size as the Multihashes produces by this derive.
       pub type Multihash = #mh_crate::MultihashGeneric::<#alloc_size>;
 
-      impl #mh_crate::MultihashDigest for #code_enum {
-          type AllocSize = #alloc_size;
-
+      impl #mh_crate::MultihashDigest<#alloc_size> for #code_enum {
           fn digest(&self, input: &[u8]) -> Multihash {
               use #mh_crate::Hasher;
               match self {
@@ -353,9 +352,8 @@ pub fn multihash(s: Structure) -> TokenStream {
               }
           }
 
-          fn multihash_from_digest<'a, S, D>(digest: &'a D) -> Multihash
+          fn multihash_from_digest<'a, D, const S: usize>(digest: &'a D) -> Multihash
           where
-              S: #mh_crate::Size,
               D: #mh_crate::Digest<S>,
               Self: From<&'a D>,
           {
@@ -396,22 +394,20 @@ mod tests {
   fn test_multihash_derive() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32)]
+       #[mh(alloc_size = 32)]
        pub enum Code {
-           #[mh(code = multihash::IDENTITY, hasher = multihash::Identity256, digest = multihash::IdentityDigest<U32>)]
+           #[mh(code = multihash::IDENTITY, hasher = multihash::Identity256, digest = multihash::IdentityDigest<32>)]
            Identity256,
            /// Multihash array for hash function.
-           #[mh(code = 0x38b64f, hasher = multihash::Strobe256, digest = multihash::StrobeDigest<U32>)]
+           #[mh(code = 0x38b64f, hasher = multihash::Strobe256, digest = multihash::StrobeDigest<32>)]
            Strobe256,
         }
     };
     let expected = quote! {
         /// A Multihash with the same allocated size as the Multihashes produces by this derive.
-        pub type Multihash = sp_multihash::MultihashGeneric::<U32>;
-
-        impl sp_multihash::MultihashDigest for Code {
-           type AllocSize = U32;
+        pub type Multihash = sp_multihash::MultihashGeneric::<32>;
 
+        impl sp_multihash::MultihashDigest<32> for Code {
            fn digest(&self, input: &[u8]) -> Multihash {
                use sp_multihash::Hasher;
                match self {
@@ -427,9 +423,8 @@ mod tests {
                }
            }
 
-           fn multihash_from_digest<'a, S, D>(digest: &'a D) -> Multihash
+           fn multihash_from_digest<'a, D, const S: usize>(digest: &'a D) -> Multihash
            where
-               S: sp_multihash::Size,
                D: sp_multihash::Digest<S>,
                Self: From<&'a D>,
            {
@@ -461,13 +456,13 @@ mod tests {
             }
         }
 
-        impl From<&multihash::IdentityDigest<U32> > for Code {
-            fn from(digest: &multihash::IdentityDigest<U32>) -> Self {
+        impl From<&multihash::IdentityDigest<32> > for Code {
+            fn from(digest: &multihash::IdentityDigest<32>) -> Self {
                 Self::Identity256
             }
         }
-        impl From<&multihash::StrobeDigest<U32> > for Code {
-            fn from(digest: &multihash::StrobeDigest<U32>) -> Self {
+        impl From<&multihash::StrobeDigest<32> > for Code {
+            fn from(digest: &multihash::StrobeDigest<32>) -> Self {
                 Self::Strobe256
             }
         }
@@ -479,17 +474,16 @@ mod tests {
   }
 
   #[test]
-  #[should_panic(
-    expected = "the #mh(code) attribute `multihash :: SHA2_256` is defined multiple times"
-  )]
+  #[should_panic(expected = "the #mh(code) attribute `multihash :: SHA2_256` \
+                             is defined multiple times")]
   fn test_multihash_error_code_duplicates() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U64)]
+       #[mh(alloc_size = 64)]
        pub enum Multihash {
-           #[mh(code = multihash::SHA2_256, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
+           #[mh(code = multihash::SHA2_256, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
            Identity256,
-           #[mh(code = multihash::SHA2_256, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
+           #[mh(code = multihash::SHA2_256, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
            Identity256,
         }
     };
@@ -499,15 +493,17 @@ mod tests {
   }
 
   #[test]
-  #[should_panic(expected = "the #mh(code) attribute `0x14` is defined multiple times")]
+  #[should_panic(
+    expected = "the #mh(code) attribute `0x14` is defined multiple times"
+  )]
   fn test_multihash_error_code_duplicates_numbers() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32)]
+       #[mh(alloc_size = 32)]
        pub enum Code {
-           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
+           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
            Identity256,
-           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
+           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
            Identity256,
         }
     };
@@ -517,12 +513,30 @@ mod tests {
   }
 
   #[test]
-  #[should_panic(expected = "enum is missing `alloc_size` attribute: e.g. #[mh(alloc_size = U64)]")]
+  #[should_panic(expected = "enum is missing `alloc_size` attribute: e.g. \
+                             #[mh(alloc_size = 64)]")]
   fn test_multihash_error_no_alloc_size() {
     let input = quote! {
        #[derive(Clone, Multihash)]
        pub enum Code {
-           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
+           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
+           Sha2_256,
+        }
+    };
+    let derive_input = syn::parse2(input).unwrap();
+    let s = Structure::new(&derive_input);
+    multihash(s);
+  }
+
+  #[test]
+  #[should_panic(expected = "The `#mh(alloc_size) attribute must be bigger \
+                             than the maximum defined digest size (32)")]
+  fn test_multihash_error_too_small_alloc_size() {
+    let input = quote! {
+       #[derive(Clone, Multihash)]
+       #[mh(alloc_size = 16)]
+       pub enum Code {
+           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<32>)]
            Sha2_256,
         }
     };
@@ -533,28 +547,12 @@ mod tests {
 
   #[test]
   #[should_panic(
-    expected = "The `#mh(alloc_size) attribute must be bigger than the maximum defined digest size (U32)"
+    expected = "Invalid byte size. It must be a unsigned integer, e.g. `32`"
   )]
-  fn test_multihash_error_too_small_alloc_size() {
-    let input = quote! {
-       #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U16)]
-       pub enum Code {
-           #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<U32>)]
-           Sha2_256,
-        }
-    };
-    let derive_input = syn::parse2(input).unwrap();
-    let s = Structure::new(&derive_input);
-    multihash(s);
-  }
-
-  #[test]
-  #[should_panic(expected = "Invalid byte size. It must be a unsigned integer typenum, e.g. `U32`")]
   fn test_multihash_error_digest_invalid_size_type() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32)]
+       #[mh(alloc_size = 32)]
        pub enum Code {
            #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<foo>)]
            Sha2_256,
@@ -566,11 +564,13 @@ mod tests {
   }
 
   #[test]
-  #[should_panic(expected = "Invalid byte size. It must be a unsigned integer typenum, e.g. `U32`")]
+  #[should_panic(
+    expected = "Invalid byte size. It must be a unsigned integer, e.g. `32`"
+  )]
   fn test_multihash_error_digest_invalid_size_type2() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32)]
+       #[mh(alloc_size = 32)]
        pub enum Code {
            #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = multihash::Sha2Digest<_>)]
            Sha2_256,
@@ -582,11 +582,13 @@ mod tests {
   }
 
   #[test]
-  #[should_panic(expected = "Invalid byte size. It must be a unsigned integer typenum, e.g. `U32`")]
-  fn test_multihash_error_digest_without_typenum() {
+  #[should_panic(
+    expected = "Invalid byte size. It must be a unsigned integer, e.g. `32`"
+  )]
+  fn test_multihash_error_digest_without_size() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32)]
+       #[mh(alloc_size = 32)]
        pub enum Code {
            #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = Sha2_256Digest)]
            Sha2_256,
@@ -599,10 +601,10 @@ mod tests {
 
   // This one does not panic, die to `no_alloc_size_errors`
   #[test]
-  fn test_multihash_error_digest_without_typenum_no_alloc_size_errors() {
+  fn test_multihash_error_digest_without_size_no_alloc_size_errors() {
     let input = quote! {
        #[derive(Clone, Multihash)]
-       #[mh(alloc_size = U32, no_alloc_size_errors)]
+       #[mh(alloc_size = 32, no_alloc_size_errors)]
        pub enum Code {
            #[mh(code = 0x14, hasher = multihash::Sha2_256, digest = Sha2_256Digest)]
            Sha2_256,
